package ui.controller;

import java.util.ArrayList;
import java.util.HashMap;

import classes.AbstractCpsObject;
import classes.HolonElement;
import classes.HolonObject;
import ui.view.DefaulTable;
import ui.view.Languages;
import ui.view.PropertyTable;
import ui.model.*;
import ui.controller.*;

/**
 * This class is for all update methods and more ;)
 * 
 * @author Edgardo
 *
 */
public class UpdateController {

	Model model;
	Control controller;

	public UpdateController(Model model, Control control) {
		this.model = model;
		this.controller = control;
	}

	/**
	 * Update the information concerning properties of the selected CpsObject.
	 */
	public void refreshTableProperties(DefaulTable table) {
		if (model.getSelectedCpsObjects().size() == 1) {
			AbstractCpsObject tempCps = model.getSelectedCpsObject();
			if (tempCps != null && tempCps.getClass() == HolonObject.class) {
				table.removeRow(2);
				Object[] tempEnergy = { Languages.getLanguage()[73], ((HolonObject) tempCps).getCurrentEnergy() };
				table.insertRow(2, tempEnergy);
			}
		}
	}

	/**
	 * Add the Information of the given ArrayList in the HolonElement Model as
	 * Name,Energy and Amount.
	 * 
	 * @param objects
	 *            ArrayList to be displayed
	 */
	public void fillElementTable(ArrayList<AbstractCpsObject> objects, PropertyTable table) {
		if (objects.size() > 1) {
			for (AbstractCpsObject o : objects) {
				if (o instanceof HolonObject) {
					for (HolonElement he : ((HolonObject) o).getElements()) {
						Object[] temp = { o.getName() + ", " + o.getID(), he.getId(), he.getEleName(), he.getEnergy(),
								he.getAmount(), he.getActive() };
						table.addRow(temp);
					}
				}
			}
		} else if (objects.size() == 1) {
			AbstractCpsObject o = objects.get(0);
			if (o instanceof HolonObject) {
				for (HolonElement he : ((HolonObject) o).getElements()) {
					Object[] temp = { he.getId(), he.getEleName(), he.getEnergy(), he.getAmount(), he.getActive() };
					table.addRow(temp);
				}
			}
		}
	}

	/**
	 * Update the HolonElement Table, that means erase all rows and add the new
	 * rows with new data.
	 */
	public void refreshTableHolonElement(PropertyTable multiTable, PropertyTable singleTable) {
		// Update of the Information about the HolonElements - only for
		// HolonObjects

		if (model.getSelectedCpsObjects().size() > 1) {
			deleteRows(multiTable);
			fillElementTable(model.getSelectedCpsObjects(), multiTable);
			multiTable.fireTableDataChanged();
		} else if (model.getSelectedCpsObjects().size() == 1) {
			deleteRows(singleTable);
			fillElementTable(model.getSelectedCpsObjects(), singleTable);
			singleTable.fireTableDataChanged();
		}
	}

	/**
	 * Erase all information of the HolonElement Model.
	 * 
	 * @param t
	 *            the Table
	 */
	public void deleteRows(PropertyTable t) {
		if (t.getRowCount() > 0) {
			for (int i = t.getRowCount() - 1; i > -1; i--) {
				t.removeRow(i);
			}
		}
	}

	/**
	 * Search for clicked HolonObject through the mouse's y-Coord.
	 * 
	 * @param yValue
	 *            the Y Coordination
	 * @return clicked HolonObject
	 */
	public HolonObject getHolonObj(int yValue, PropertyTable table) {
		final int yTemp = (int) Math.floor(yValue / 16);
		HolonObject obtTemp = null;
		String temp = table.getValueAt(yTemp, 0).toString();
		int idTemp = Integer.parseInt(temp.split(", ")[1]);
		obtTemp = (HolonObject) controller.searchByID(idTemp);
		return obtTemp;
	}

	/**
	 * Search for actual selected HolonElement.
	 * 
	 * @param obj
	 *            selected HolonObject, if obj==null means multi-selection
	 *            active
	 * @param yValue
	 *            Y-Coord in the HolonElementsTable
	 * @param toMultiHash
	 *            0 means no MultiSelection, 1 means MultiSelection without
	 *            Control, 2 means MultiSelection with Control
	 * @return the selected HolonElement
	 */
	public HolonElement getActualHolonElement(HolonObject obj, int yValue, int toMultiHash,
			ArrayList<PropertyTable> tables) {
		final int yTemp = (int) Math.floor(yValue / 16);
		int rowsTotal = 0;
		// Filter for search --> single and multi selection
		if (obj == null) {
			rowsTotal = tables.get(1).getRowCount();
		} else {
			rowsTotal = tables.get(0).getRowCount();
		}
		// search for the clicked HolonObject and HolonElement --> in the
		// HolonElementTable
		HolonObject obtTemp = null;
		HolonElement toReturnEle = null;
		int id = 0;
		if (rowsTotal != 0 && rowsTotal > yTemp) {
			// Multi-Selection search
			if (obj == null) {
				String tempStringObj = tables.get(1).getValueAt(yTemp, 0).toString();
				int idTempObj = Integer.parseInt(tempStringObj.split(", ")[1]);
				obtTemp = (HolonObject) controller.searchByID(idTempObj);
				id = Integer.parseInt(tables.get(1).getValueAt(yTemp, 1).toString());
				ArrayList<HolonElement> eleTemp = new ArrayList<HolonElement>();
				if (model.getEleToDelete().containsKey(idTempObj) && toMultiHash == 2) {
					eleTemp = model.getEleToDelete().get(idTempObj);
					if (!eleTemp.contains(obtTemp.searchElementById(id))) {
						eleTemp.add(obtTemp.searchElementById(id));
						model.getEleToDelete().replace(idTempObj, eleTemp);
					}
				} else if (toMultiHash == 2) {
					eleTemp.add(obtTemp.searchElementById(id));
					model.getEleToDelete().put(idTempObj, eleTemp);
				} else if (toMultiHash == 1) {
					model.setEleToDelete(new HashMap<Integer, ArrayList<HolonElement>>());
					eleTemp.add(obtTemp.searchElementById(id));
					model.getEleToDelete().put(idTempObj, eleTemp);
				} else if (toMultiHash == 0) {
					toReturnEle = obtTemp.searchElementById(id);
				}
			} // Single-Selection search
			else {
				model.setEleToDelete(new HashMap<Integer, ArrayList<HolonElement>>());
				id = Integer.parseInt(tables.get(0).getValueAt(yTemp, 0).toString());
				toReturnEle = obj.searchElementById(id);
			}
			model.setSelectedHolonElement(toReturnEle);
			return toReturnEle;
		} // If no HolonObject selected
		else {
			model.setEleToDelete(new HashMap<Integer, ArrayList<HolonElement>>());
			model.setSelectedHolonElement(null);
			return null;
		}
	}

	/**
	 * Getter for selected CpsObject.
	 * 
	 * @return selected CpsObject
	 */
	public AbstractCpsObject getActualCps() {
		int tempID = model.getSelectedObjectID();
		AbstractCpsObject tempCps = controller.searchByID(tempID);
		return tempCps;
	}
}
